"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var cachedTagFilterFunctions = {};
var convertTagFilterExpressionToFunction = function (tagFilterExpression) {
    var tagRegex = /(\@[A-Za-z-_0-9]+)/g;
    var tags = [];
    var match = null;
    var newTagFilterExpression = tagFilterExpression + '';
    do {
        match = tagRegex.exec(tagFilterExpression);
        if (match) {
            // tslint:disable-next-line:max-line-length
            newTagFilterExpression = newTagFilterExpression.replace(match[1], "(tags.indexOf(\"" + match[1].toLowerCase() + "\")!==-1)");
            if (tags.indexOf(match[1]) !== -1) {
                tags.push(match[1]);
            }
        }
    } while (match);
    newTagFilterExpression = newTagFilterExpression.replace(/(\s+not|not\s+|\s+not\s+)/g, ' ! ');
    newTagFilterExpression = newTagFilterExpression.replace(/(\s+or|or\s+|\s+or\s+)/g, ' || ');
    newTagFilterExpression = newTagFilterExpression.replace(/(\s+and|and\s+|\s+and\s+)/g, ' && ');
    newTagFilterExpression = newTagFilterExpression.replace(/[ \t\n\r]+/g, '');
    var tagFilterFunction;
    try {
        tagFilterFunction = new Function('tags', "return " + newTagFilterExpression + ";");
        var result = tagFilterFunction([]);
        if (result !== true && result !== false) {
            throw new Error();
        }
    }
    catch (error) {
        throw new Error("Could not parse tag filter \"" + tagFilterExpression + "\"");
    }
    return tagFilterFunction;
};
var checkIfScenarioMatchesTagFilter = function (tagFilterExpression, feature, scenario) {
    var featureAndScenarioTags = scenario.tags.map(function (tag) { return tag.toLowerCase(); }).concat(feature.tags.map(function (tag) { return tag.toLowerCase(); }));
    var tagFilterFunction = cachedTagFilterFunctions[tagFilterExpression];
    if (!tagFilterFunction) {
        tagFilterFunction = convertTagFilterExpressionToFunction(tagFilterExpression);
        cachedTagFilterFunctions[tagFilterExpression] = tagFilterFunction;
    }
    return tagFilterFunction(featureAndScenarioTags) === true;
};
var setScenarioSkipped = function (parsedFeature, scenario) {
    var skippedViaTagFilter = !checkIfScenarioMatchesTagFilter(parsedFeature.options.tagFilter, parsedFeature, scenario);
    return __assign({}, scenario, { skippedViaTagFilter: skippedViaTagFilter });
};
exports.applyTagFilters = function (parsedFeature) {
    if (parsedFeature.options.tagFilter === undefined) {
        return parsedFeature;
    }
    var scenarios = parsedFeature.scenarios.map(function (scenario) { return setScenarioSkipped(parsedFeature, scenario); });
    var scenarioOutlines = parsedFeature.scenarioOutlines
        .map(function (scenarioOutline) {
        return __assign({}, setScenarioSkipped(parsedFeature, scenarioOutline), { scenarios: scenarioOutline.scenarios.map(function (scenario) { return setScenarioSkipped(parsedFeature, scenario); }) });
    });
    return __assign({}, parsedFeature, { scenarios: scenarios,
        scenarioOutlines: scenarioOutlines });
};
//# sourceMappingURL=tag-filtering.js.map